@@ -20,9 +20,10 @@ from __future__ import absolute_import, division, print_function
 from collections import defaultdict
 
 import hypothesis.internal.conjecture.utils as cu
-from hypothesis._settings import Phase, note_deprecation
+from hypothesis._settings import Phase
 from hypothesis.control import _current_build_context, assume
 from hypothesis.errors import (
+    HypothesisException,
     NoExamples,
     NoSuchExample,
     Unsatisfiable,
@@ -126,6 +127,7 @@ class SearchStrategy(Generic[Ex]):
                 pass
 
             mapping = {}
+            sentinel = object()
             hit_recursion = [False]
 
             # For a first pass we do a direct recursive calculation of the
@@ -137,17 +139,15 @@ class SearchStrategy(Generic[Ex]):
                     return forced_value(strat)
                 except AttributeError:
                     pass
-                try:
-                    result = mapping[strat]
-                    if result is calculating:
-                        hit_recursion[0] = True
-                        return default
-                    else:
-                        return result
-                except KeyError:
+                result = mapping.get(strat, sentinel)
+                if result is calculating:
+                    hit_recursion[0] = True
+                    return default
+                elif result is sentinel:
                     mapping[strat] = calculating
                     mapping[strat] = getattr(strat, calculation)(recur)
                     return mapping[strat]
+                return result
 
             recur(self)
 
@@ -176,12 +176,12 @@ class SearchStrategy(Generic[Ex]):
                     except AttributeError:
                         pass
                     listeners[other].add(strat)
-                    try:
-                        return mapping[other]
-                    except KeyError:
+                    result = mapping.get(other, sentinel)
+                    if result is sentinel:
                         needs_update.add(other)
                         mapping[other] = default
                         return default
+                    return result
 
                 return recur_inner
 
@@ -266,31 +266,22 @@ class SearchStrategy(Generic[Ex]):
         context = _current_build_context.value
         if context is not None:
             if context.data is not None and context.data.depth > 0:
-                note_deprecation(
+                raise HypothesisException(
                     "Using example() inside a strategy definition is a bad "
-                    "idea. It will become an error in a future version of "
-                    "Hypothesis, but it's unlikely that it's doing what you "
-                    "intend even now. Instead consider using "
-                    "hypothesis.strategies.builds() or "
-                    "@hypothesis.strategies.composite to define your strategy."
-                    " See "
-                    "https://hypothesis.readthedocs.io/en/latest/data.html"
+                    "idea. Instead consider using hypothesis.strategies.builds() "
+                    "or @hypothesis.strategies.composite to define your strategy."
+                    " See https://hypothesis.readthedocs.io/en/latest/data.html"
                     "#hypothesis.strategies.builds or "
                     "https://hypothesis.readthedocs.io/en/latest/data.html"
-                    "#composite-strategies for more details.",
-                    since="2017-09-04",
+                    "#composite-strategies for more details."
                 )
             else:
-                note_deprecation(
+                raise HypothesisException(
                     "Using example() inside a test function is a bad "
-                    "idea. It will become an error in a future version of "
-                    "Hypothesis, but it's unlikely that it's doing what you "
-                    "intend even now. Instead consider using "
-                    "hypothesis.strategies.data() to draw "
-                    "more examples during testing. See "
+                    "idea. Instead consider using hypothesis.strategies.data() "
+                    "to draw more examples during testing. See "
                     "https://hypothesis.readthedocs.io/en/latest/data.html"
-                    "#drawing-interactively-in-tests for more details.",
-                    since="2017-09-04",
+                    "#drawing-interactively-in-tests for more details."
                 )
 
         from hypothesis import find, settings, Verbosity
@@ -355,7 +346,13 @@ class SearchStrategy(Generic[Ex]):
 
         This method is part of the public API.
         """
-        return FilteredStrategy(condition=condition, strategy=self)
+        return FilteredStrategy(conditions=(condition,), strategy=self)
+
+    def do_filtered_draw(self, data, filter_strategy):
+        # Hook for strategies that want to override the behaviour of
+        # FilteredStrategy. Most strategies don't, so by default we delegate
+        # straight back to the default filtered-draw implementation.
+        return filter_strategy.default_do_filtered_draw(data)
 
     @property
     def branches(self):
@@ -434,24 +431,12 @@ class OneOfStrategy(SearchStrategy):
     conditional distribution of that strategy.
     """
 
-    def __init__(self, strategies, bias=None):
+    def __init__(self, strategies):
         SearchStrategy.__init__(self)
         strategies = tuple(strategies)
         self.original_strategies = list(strategies)
         self.__element_strategies = None
-        self.bias = bias
         self.__in_branches = False
-        self.__sampler = None
-
-    @property
-    def sampler(self):
-        if self.__sampler is None:
-            assert self.bias is not None
-            assert 0 < self.bias < 1
-            self.__sampler = cu.Sampler(
-                [self.bias ** i for i in range(len(self.element_strategies))]
-            )
-        return self.__sampler
 
     def calc_is_empty(self, recur):
         return all(recur(e) for e in self.original_strategies)
@@ -507,11 +492,7 @@ class OneOfStrategy(SearchStrategy):
         if n == 1:
             return data.draw(self.element_strategies[0])
 
-        if self.bias is None:
-            i = cu.integer_range(data, 0, n - 1)
-        else:
-            i = self.sampler.sample(data)
-            assert 0 <= i < n
+        i = cu.integer_range(data, 0, n - 1)
 
         return data.draw(self.element_strategies[i], label=self.branch_labels[i])
 
@@ -524,7 +505,7 @@ class OneOfStrategy(SearchStrategy):
 
     @property
     def branches(self):
-        if self.bias is None and not self.__in_branches:
+        if not self.__in_branches:
             try:
                 self.__in_branches = True
                 return self.element_strategies
@@ -593,11 +574,26 @@ class MappedSearchStrategy(SearchStrategy):
         ]
 
 
+filter_not_satisfied = UniqueIdentifier("filter not satisfied")
+
+
 class FilteredStrategy(SearchStrategy):
-    def __init__(self, strategy, condition):
+    def __init__(self, strategy, conditions):
         super(FilteredStrategy, self).__init__()
-        self.condition = condition
-        self.filtered_strategy = strategy
+        if isinstance(strategy, FilteredStrategy):
+            # Flatten chained filters into a single filter with multiple
+            # conditions.
+            self.flat_conditions = strategy.flat_conditions + conditions
+            self.filtered_strategy = strategy.filtered_strategy
+        else:
+            self.flat_conditions = conditions
+            self.filtered_strategy = strategy
+
+        assert self.flat_conditions
+        assert isinstance(self.flat_conditions, tuple)
+        assert not isinstance(self.filtered_strategy, FilteredStrategy)
+
+        self.__condition = None
 
     def calc_is_empty(self, recur):
         return recur(self.filtered_strategy)
@@ -607,17 +603,48 @@ class FilteredStrategy(SearchStrategy):
 
     def __repr__(self):
         if not hasattr(self, "_cached_repr"):
-            self._cached_repr = "%r.filter(%s)" % (
+            self._cached_repr = "%r%s" % (
                 self.filtered_strategy,
-                get_pretty_function_description(self.condition),
+                "".join(
+                    ".filter(%s)" % get_pretty_function_description(cond)
+                    for cond in self.flat_conditions
+                ),
             )
         return self._cached_repr
 
     def do_validate(self):
         self.filtered_strategy.validate()
 
+    @property
+    def condition(self):
+        if self.__condition is None:
+            assert self.flat_conditions
+            if len(self.flat_conditions) == 1:
+                # Avoid an extra indirection in the common case of only one
+                # condition.
+                self.__condition = self.flat_conditions[0]
+            else:
+                self.__condition = lambda x: all(
+                    cond(x) for cond in self.flat_conditions
+                )
+        return self.__condition
+
     def do_draw(self, data):
         # type: (ConjectureData) -> Ex
+        result = self.filtered_strategy.do_filtered_draw(
+            data=data, filter_strategy=self
+        )
+        if result is not filter_not_satisfied:
+            return result
+
+        data.note_event("Aborted test because unable to satisfy %r" % (self,))
+        data.mark_invalid()
+        raise AssertionError("Unreachable, for Mypy")  # pragma: no cover
+
+    def note_retried(self, data):
+        data.note_event(lazyformat("Retried draw from %r to satisfy filter", self))
+
+    def default_do_filtered_draw(self, data):
         for i in hrange(3):
             start_index = data.index
             value = data.draw(self.filtered_strategy)
@@ -625,22 +652,19 @@ class FilteredStrategy(SearchStrategy):
                 return value
             else:
                 if i == 0:
-                    data.note_event(
-                        lazyformat("Retried draw from %r to satisfy filter", self)
-                    )
+                    self.note_retried(data)
                 # This is to guard against the case where we consume no data.
                 # As long as we consume data, we'll eventually pass or raise.
                 # But if we don't this could be an infinite loop.
                 assume(data.index > start_index)
-        data.note_event("Aborted test because unable to satisfy %r" % (self,))
-        data.mark_invalid()
-        raise AssertionError("Unreachable, for Mypy")  # pragma: no cover
+
+        return filter_not_satisfied
 
     @property
     def branches(self):
         # type: () -> List[SearchStrategy[Ex]]
         return [
-            FilteredStrategy(strategy=strategy, condition=self.condition)
+            FilteredStrategy(strategy=strategy, conditions=self.flat_conditions)
             for strategy in self.filtered_strategy.branches
         ]
 
